@@ -52,12 +52,11 @@
  * you can set #GstVaapiEncodeH264:tune, if your backend supports it,
  * for low-power mode or high compression.
  *
- * <refsect2>
- * <title>Example launch line</title>
+ * ## Example launch line
+ *
  * |[
  *  gst-launch-1.0 -ev videotestsrc num-buffers=60 ! timeoverlay ! vaapih264enc ! h264parse ! mp4mux ! filesink location=test.mp4
  * ]|
- * </refsect2>
  */
 
 #include "gstcompat.h"
@@ -79,43 +78,16 @@ GST_DEBUG_CATEGORY_STATIC (gst_vaapi_h264_encode_debug);
   "stream-format = (string) { avc, byte-stream }, " \
   "alignment = (string) au"
 
-/* *INDENT-OFF* */
-static const char gst_vaapiencode_h264_sink_caps_str[] =
-  GST_VAAPI_MAKE_SURFACE_CAPS ", "
-  GST_CAPS_INTERLACED_FALSE "; "
-  GST_VIDEO_CAPS_MAKE (GST_VIDEO_FORMATS_ALL) ", "
-  GST_CAPS_INTERLACED_FALSE;
-/* *INDENT-ON* */
-
-/* *INDENT-OFF* */
-static const char gst_vaapiencode_h264_src_caps_str[] =
-  GST_CODEC_CAPS ", "
-  "profile = (string) { constrained-baseline, baseline, main, high, multiview-high, stereo-high }";
-/* *INDENT-ON* */
-
-/* *INDENT-OFF* */
-static GstStaticPadTemplate gst_vaapiencode_h264_sink_factory =
-  GST_STATIC_PAD_TEMPLATE ("sink",
-      GST_PAD_SINK,
-      GST_PAD_ALWAYS,
-      GST_STATIC_CAPS (gst_vaapiencode_h264_sink_caps_str));
-/* *INDENT-ON* */
-
-/* *INDENT-OFF* */
-static GstStaticPadTemplate gst_vaapiencode_h264_src_factory =
-  GST_STATIC_PAD_TEMPLATE ("src",
-      GST_PAD_SRC,
-      GST_PAD_ALWAYS,
-      GST_STATIC_CAPS (gst_vaapiencode_h264_src_caps_str));
-/* *INDENT-ON* */
+#define EXTRA_FORMATS {}
 
 /* h264 encode */
-G_DEFINE_TYPE (GstVaapiEncodeH264, gst_vaapiencode_h264, GST_TYPE_VAAPIENCODE);
+GST_VAAPI_ENCODE_REGISTER_TYPE (h264, H264, H264, EXTRA_FORMATS,
+    gst_vaapi_utils_h264_get_profile_string);
 
 static void
 gst_vaapiencode_h264_init (GstVaapiEncodeH264 * encode)
 {
-  gst_vaapiencode_init_properties (GST_VAAPIENCODE_CAST (encode));
+  /* nothing to do here */
 }
 
 static void
@@ -127,53 +99,12 @@ gst_vaapiencode_h264_finalize (GObject * object)
   G_OBJECT_CLASS (gst_vaapiencode_h264_parent_class)->finalize (object);
 }
 
-static void
-gst_vaapiencode_h264_set_property (GObject * object,
-    guint prop_id, const GValue * value, GParamSpec * pspec)
+static GArray *
+gst_vaapiencode_h264_get_allowed_profiles (GstVaapiEncode * encode,
+    GstCaps * allowed)
 {
-  GstVaapiEncodeClass *const encode_class = GST_VAAPIENCODE_GET_CLASS (object);
-  GstVaapiEncode *const base_encode = GST_VAAPIENCODE_CAST (object);
-
-  switch (prop_id) {
-    default:
-      if (!encode_class->set_property (base_encode, prop_id, value))
-        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-      break;
-  }
-}
-
-static void
-gst_vaapiencode_h264_get_property (GObject * object,
-    guint prop_id, GValue * value, GParamSpec * pspec)
-{
-  GstVaapiEncodeClass *const encode_class = GST_VAAPIENCODE_GET_CLASS (object);
-  GstVaapiEncode *const base_encode = GST_VAAPIENCODE_CAST (object);
-
-  switch (prop_id) {
-    default:
-      if (!encode_class->get_property (base_encode, prop_id, value))
-        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-      break;
-  }
-}
-
-static GstVaapiProfile
-gst_vaapiencode_h264_get_profile (GstCaps * caps)
-{
-  guint i;
-
-  for (i = 0; i < gst_caps_get_size (caps); i++) {
-    GstStructure *const structure = gst_caps_get_structure (caps, i);
-    const GValue *const value = gst_structure_get_value (structure, "profile");
-
-    if (value && G_VALUE_HOLDS_STRING (value)) {
-      const gchar *str = g_value_get_string (value);
-      if (str)
-        return gst_vaapi_utils_h264_get_profile_from_string (str);
-    }
-  }
-
-  return GST_VAAPI_PROFILE_UNKNOWN;
+  return gst_vaapi_encoder_get_profiles_from_caps (allowed,
+      gst_vaapi_utils_h264_get_profile_from_string);
 }
 
 typedef struct
@@ -235,6 +166,9 @@ find_best_profile (GstCaps * caps)
 static GstCaps *
 get_available_caps (GstVaapiEncodeH264 * encode)
 {
+  GstVaapiEncode *const base_encode = GST_VAAPIENCODE_CAST (encode);
+  GstVaapiEncoderH264 *const encoder =
+      GST_VAAPI_ENCODER_H264 (base_encode->encoder);
   GstCaps *out_caps;
   GArray *profiles;
   GstVaapiProfile profile;
@@ -272,6 +206,13 @@ get_available_caps (GstVaapiEncodeH264 * encode)
   g_value_unset (&profile_list);
   g_value_unset (&profile_v);
 
+  if (!gst_vaapi_encoder_h264_supports_avc (encoder)) {
+    GST_INFO_OBJECT (encode,
+        "avc requires packed header support, outputting byte-stream");
+    gst_caps_set_simple (out_caps, "stream-format", G_TYPE_STRING,
+        "byte-stream", NULL);
+  }
+
   encode->available_caps = out_caps;
 
   return encode->available_caps;
@@ -287,7 +228,7 @@ gst_vaapiencode_h264_set_config (GstVaapiEncode * base_encode)
   gboolean ret = TRUE;
 
   template_caps =
-      gst_static_pad_template_get_caps (&gst_vaapiencode_h264_src_factory);
+      gst_pad_get_pad_template_caps (GST_VAAPI_PLUGIN_BASE_SRC_PAD (encode));
   allowed_caps =
       gst_pad_get_allowed_caps (GST_VAAPI_PLUGIN_BASE_SRC_PAD (encode));
 
@@ -301,47 +242,58 @@ gst_vaapiencode_h264_set_config (GstVaapiEncode * base_encode)
     encode->is_avc = FALSE;
   } else if (gst_caps_is_empty (allowed_caps)) {
     GST_INFO_OBJECT (encode, "downstream has EMPTY caps");
-    gst_caps_unref (template_caps);
-    gst_caps_unref (allowed_caps);
-    return FALSE;
+    goto fail;
   } else {
     const char *stream_format = NULL;
     GstStructure *structure;
-    guint i, num_structures;
     GstVaapiProfile profile = GST_VAAPI_PROFILE_UNKNOWN;
     GstCaps *available_caps;
+    GstCaps *profile_caps;
 
     available_caps = get_available_caps (encode);
-    if (!available_caps) {
-      gst_caps_unref (template_caps);
-      gst_caps_unref (allowed_caps);
-      return FALSE;
-    }
+    if (!available_caps)
+      goto fail;
+
     if (!gst_caps_can_intersect (allowed_caps, available_caps)) {
-      GST_INFO_OBJECT (encode, "downstream requested an unsupported profile, "
-          "but encoder will try to output a compatible one");
+      GstCaps *tmp_caps;
+
+      GST_INFO_OBJECT (encode, "downstream may have requested an unsupported "
+          "profile. Encoder will try to output a compatible one");
 
       /* Let's try the best profile in the allowed caps.
        * The internal encoder will fail later if it can't handle it */
       profile = find_best_profile (allowed_caps);
+      if (profile == GST_VAAPI_PROFILE_UNKNOWN)
+        goto fail;
 
+      /* if allwed caps request baseline (which is deprecated), the
+       * encoder will try with constrained baseline which is
+       * compatible. */
+      if (profile == GST_VAAPI_PROFILE_H264_BASELINE)
+        profile = GST_VAAPI_PROFILE_H264_CONSTRAINED_BASELINE;
+
+      tmp_caps = gst_caps_from_string (GST_CODEC_CAPS);
+      gst_caps_set_simple (tmp_caps, "profile", G_TYPE_STRING,
+          gst_vaapi_profile_get_name (profile), NULL);
+      if (!gst_vaapi_encoder_h264_supports_avc (encoder)) {
+        gst_caps_set_simple (tmp_caps, "stream-format", G_TYPE_STRING,
+            "byte-stream", NULL);
+      }
+
+      profile_caps = gst_caps_intersect (available_caps, tmp_caps);
+      gst_caps_unref (tmp_caps);
+      if (gst_caps_is_empty (profile_caps)) {
+        gst_caps_unref (profile_caps);
+        goto fail;
+      }
     } else {
-      GstCaps *profile_caps;
       profile_caps = gst_caps_intersect (allowed_caps, available_caps);
       profile = find_best_profile (profile_caps);
-
-      gst_caps_unref (profile_caps);
     }
 
-    /* Check whether "stream-format" is avcC mode */
-    num_structures = gst_caps_get_size (allowed_caps);
-    for (i = 0; !stream_format && i < num_structures; i++) {
-      structure = gst_caps_get_structure (allowed_caps, i);
-      if (!gst_structure_has_field_typed (structure, "stream-format",
-              G_TYPE_STRING))
-        continue;
-      stream_format = gst_structure_get_string (structure, "stream-format");
-    }
+    profile_caps = gst_caps_fixate (profile_caps);
+    structure = gst_caps_get_structure (profile_caps, 0);
+    stream_format = gst_structure_get_string (structure, "stream-format");
     encode->is_avc = (g_strcmp0 (stream_format, "avc") == 0);
 
     if (profile != GST_VAAPI_PROFILE_UNKNOWN) {
@@ -352,6 +304,7 @@ gst_vaapiencode_h264_set_config (GstVaapiEncode * base_encode)
       ret = FALSE;
     }
 
+    gst_caps_unref (profile_caps);
     gst_caps_unref (allowed_caps);
   }
   gst_caps_unref (template_caps);
@@ -359,11 +312,18 @@ gst_vaapiencode_h264_set_config (GstVaapiEncode * base_encode)
   base_encode->need_codec_data = encode->is_avc;
 
   return ret;
+
+fail:
+  {
+    gst_caps_unref (template_caps);
+    gst_caps_unref (allowed_caps);
+    return FALSE;
+  }
 }
 
 static void
 set_compatible_profile (GstVaapiEncodeH264 * encode, GstCaps * caps,
-    GstVaapiProfile profile)
+    GstVaapiProfile profile, GstVaapiLevelH264 level)
 {
   GstCaps *allowed_caps, *tmp_caps;
   gboolean ret = FALSE;
@@ -393,16 +353,18 @@ retry:
   if (!gst_caps_can_intersect (allowed_caps, tmp_caps)) {
     if (profile == GST_VAAPI_PROFILE_H264_CONSTRAINED_BASELINE) {
       profile = GST_VAAPI_PROFILE_H264_BASELINE;
+      GST_WARNING_OBJECT (GST_VAAPIENCODE_CAST (encode), "user might requested "
+          "baseline profile, trying constrained-baseline instead");
       goto retry;
     }
   } else {
     gst_caps_set_simple (caps, "profile", G_TYPE_STRING,
-        gst_vaapi_utils_h264_get_profile_string (profile), NULL);
+        gst_vaapi_utils_h264_get_profile_string (profile),
+        "level", G_TYPE_STRING, gst_vaapi_utils_h264_get_level_string (level),
+        NULL);
     ret = TRUE;
   }
 
-  GST_INFO_OBJECT (encode, "out caps %" GST_PTR_FORMAT, caps);
-
   if (!ret)
     GST_LOG ("There is no compatible profile in the requested caps.");
 
@@ -418,6 +380,7 @@ gst_vaapiencode_h264_get_caps (GstVaapiEncode * base_encode)
   GstVaapiEncoderH264 *const encoder =
       GST_VAAPI_ENCODER_H264 (base_encode->encoder);
   GstVaapiProfile profile;
+  GstVaapiLevelH264 level;
   GstCaps *caps;
 
   caps = gst_caps_from_string (GST_CODEC_CAPS);
@@ -426,11 +389,11 @@ gst_vaapiencode_h264_get_caps (GstVaapiEncode * base_encode)
       encode->is_avc ? "avc" : "byte-stream", NULL);
 
   /* Update profile determined by encoder */
-  gst_vaapi_encoder_h264_get_profile_and_level (encoder, &profile, NULL);
+  gst_vaapi_encoder_h264_get_profile_and_level (encoder, &profile, &level);
   if (profile != GST_VAAPI_PROFILE_UNKNOWN)
-    set_compatible_profile (encode, caps, profile);
+    set_compatible_profile (encode, caps, profile, level);
 
-  /* XXX: update level information */
+  GST_INFO_OBJECT (base_encode, "out caps %" GST_PTR_FORMAT, caps);
   return caps;
 }
 
@@ -571,21 +534,23 @@ error_convert_buffer:
 }
 
 static void
-gst_vaapiencode_h264_class_init (GstVaapiEncodeH264Class * klass)
+gst_vaapiencode_h264_class_init (GstVaapiEncodeH264Class * klass, gpointer data)
 {
   GObjectClass *const object_class = G_OBJECT_CLASS (klass);
   GstElementClass *const element_class = GST_ELEMENT_CLASS (klass);
   GstVaapiEncodeClass *const encode_class = GST_VAAPIENCODE_CLASS (klass);
-
-  GST_DEBUG_CATEGORY_INIT (gst_vaapi_h264_encode_debug,
-      GST_PLUGIN_NAME, 0, GST_PLUGIN_DESC);
+  GstCaps *sink_caps = ((GstVaapiEncodeInitData *) data)->sink_caps;
+  GstCaps *src_caps = ((GstVaapiEncodeInitData *) data)->src_caps;
+  GstPadTemplate *templ;
+  GstCaps *static_caps;
+  gpointer encoder_class;
 
   object_class->finalize = gst_vaapiencode_h264_finalize;
-  object_class->set_property = gst_vaapiencode_h264_set_property;
-  object_class->get_property = gst_vaapiencode_h264_get_property;
+  object_class->set_property = gst_vaapiencode_set_property_subclass;
+  object_class->get_property = gst_vaapiencode_get_property_subclass;
 
-  encode_class->get_properties = gst_vaapi_encoder_h264_get_default_properties;
-  encode_class->get_profile = gst_vaapiencode_h264_get_profile;
+  encode_class->get_allowed_profiles =
+      gst_vaapiencode_h264_get_allowed_profiles;
   encode_class->set_config = gst_vaapiencode_h264_set_config;
   encode_class->get_caps = gst_vaapiencode_h264_get_caps;
   encode_class->alloc_encoder = gst_vaapiencode_h264_alloc_encoder;
@@ -597,12 +562,26 @@ gst_vaapiencode_h264_class_init (GstVaapiEncodeH264Class * klass)
       GST_PLUGIN_DESC, "Wind Yuan <feng.yuan@intel.com>");
 
   /* sink pad */
-  gst_element_class_add_static_pad_template (element_class,
-      &gst_vaapiencode_h264_sink_factory);
+  g_assert (sink_caps);
+  static_caps = gst_caps_from_string (GST_VAAPI_ENCODE_STATIC_SINK_CAPS);
+  templ =
+      gst_pad_template_new ("sink", GST_PAD_SINK, GST_PAD_ALWAYS, sink_caps);
+  gst_pad_template_set_documentation_caps (templ, static_caps);
+  gst_element_class_add_pad_template (element_class, templ);
+  gst_caps_unref (static_caps);
+  gst_caps_unref (sink_caps);
 
   /* src pad */
-  gst_element_class_add_static_pad_template (element_class,
-      &gst_vaapiencode_h264_src_factory);
+  g_assert (src_caps);
+  static_caps = gst_caps_from_string (GST_CODEC_CAPS);
+  templ = gst_pad_template_new ("src", GST_PAD_SRC, GST_PAD_ALWAYS, src_caps);
+  gst_pad_template_set_documentation_caps (templ, static_caps);
+  gst_element_class_add_pad_template (element_class, templ);
+  gst_caps_unref (static_caps);
+  gst_caps_unref (src_caps);
 
-  gst_vaapiencode_class_init_properties (encode_class);
+  encoder_class = g_type_class_ref (GST_TYPE_VAAPI_ENCODER_H264);
+  g_assert (encoder_class);
+  gst_vaapiencode_class_install_properties (encode_class, encoder_class);
+  g_type_class_unref (encoder_class);
 }
